/*-------------------------------------------------------- File Description -*\

  Copyright (C) Odin TeleSystems Inc., 1994-1996

  NAME
    BALDER.H

  DESCRIPTION
    Header containing the Application Programming Interface (API) for
    Odin TeleSystems' Balder ISDN interface board.

    This header should used in conjunction with the Balder driver library,
    BLD.LIB (large memory model only).

  COMPILERS
    Borland C++, Version 4.53
    Microsoft C, Version 5.1

  SEE ALSO
    Balder board, Installation Guide.


  Copyright (C) Odin TeleSystems Inc., 1994-1996

  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

\*--------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*\
  NAMING CONVENTIONS
    All the types and functions used in this API have a prefix 'Bld' (For
    Balder).

    The function names consist of three parts: the header 'Bld', operation,
    and target. The operation word is verb describing the action to be taken,
    e.g., construct, read. The target can be either 'Driver' or 'Li'. 'Driver'
    functions have effect on the whole system, when 'Li' function only affect
    specified Line Interfaces.

  NUMBERING OF LI'S
    The balder boards should be numbered from 0 to MAX_NO_OF_BOARDS.
    Each balder board contains 2 line Interfaces (ISAC-S chips).
    The LIs are numbered consequtevily from 0 to 2*MAX_NO_OF_BOARDS-1.
    For example, if we have three boards, the numbering is as follows:
        BOARD #         LI #
        --------------------
           0            0
                        1
        --------------------
           1            2
                        3
        --------------------
           2            4
                        5
        --------------------

  EXCEPTION HANDLING
    Exception handling is taken care with return codes. Each function returns
    0 if successful (no errors) and an other non-zero value in the case of a
    problem. The return codes can be translated into strings containing the
    error message with the bldGetErrMsg() function.
\*---------------------------------------------------------------------------*/
#ifndef BALDER_H
#define BALDER_H

#ifdef __cplusplus
  extern "C" {
#endif


/*---------------------------------------------------------------------------*\
  MACRO DEFINITIONS
\*---------------------------------------------------------------------------*/



/* Frame End Codes for fmStatus in BldFrameHeader */
#define BLD_FEC_OK            0x00 /* Good message (no errors)               */
#define BLD_FEC_ABORTED       0x01 /* Set if the message was aborted by the
                                   .. remote side, i.e. a sequence of 7 1's
                                   .. was detected                           */
#define BLD_FEC_BAD_CRC       0x02 /* Set if CRC is bad                      */
#define BLD_FEC_OVERRUN       0x04 /* Set if at least one byte of the frame
                                       .. has been lost due hardware fifo
                                       .. overflow (PC too slow)             */

#define BLD_FEC_TRUNCATED     0x10 /* Set if message is longer than the
                                   .. buffer supplied by the application
                                   .. program                                */
#define BLD_FEC_OVERFLOW      0x20 /* Set if the application is too slow
                                   .. reading messages from the driver.
                                   .. Messages lost                          */


/*---------------------------------------------------------------------------*\
  TYPE DEFINITIONS
\*---------------------------------------------------------------------------*/

typedef enum {
/*---------------------------------------------------------------------------*\
  Return codes for all of the BldXxxxx functions.

  (Design Note: Do not update the number without updating
  bldErrorStr[])
\*---------------------------------------------------------------------------*/
  BLD_UNDEFINED          =  0,  /* The driver does not know what to return
                                .. for this request                          */
  BLD_SUCCESS            =  1,  /* All OK, no errors.                        */
  BLD_INVALID_MODE       =  2,  /* Supplied Li mode is not allowed.          */
  BLD_INVALID_BOARD_TYPE =  3,  /* Supplied Board type is not supported by
                                .. by this driver.                           */
  BLD_INVALID_BOARD_NO   =  4,  /* Driver was supplied a Board number which
                                .. is not allowed (should be 0<=NO<=3).      */
  BLD_INVALID_LI_NO      =  5,  /* Driver was supplied a Li number which
                                .. is not allowed (should be 0<=NO<=8).      */
  BLD_INVALID_B_CHANNEL  =  6,  /* Driver was supplied an illegal  B channel
                                .. number (should be 1<=NO<=2).              */
  BLD_INVALID_IRQ_NO     =  7,  /* Driver was supplied an IRQ number which
                                .. is not allowed (Not supported by
                                .. the board)                                */
  BLD_INVALID_ADDR       =  8,  /* Driver was supplied an I/O-address which
                                .. is not allowed (Not supported by the
                                .. Board)                                    */
  BLD_NOT_SETUP          =  9,  /* Functions has been called without a proper
                                .. configuration of the driver.              */
  BLD_NO_CLOCKS          = 10,  /* Could not enable clocks for the ISAC
                                .. functions.  (??? is this used)            */
  BLD_NO_FRAMES          = 11,  /* No full frames received and ready for
                                .. reading.                                  */
  BLD_TX_BUSY            = 12,  /* Transmitter not ready. Transmission of the
                                .. previous frame has not been completed.    */
  BLD_TIMEOUT            = 13,  /* Function timed out before the expected
                                .. response was obtained from ISR or ISAC    */
  BLD_L1_OK              = 14,  /* Physical Layer (L1) is up.                */
  BLD_L1_DOWN            = 15,  /* Physical Layer (L1) is down,
                                .. no connection.                            */
  BLD_BAD_CHIP           = 16,  /* Line Interface (ISAC-S chip) did not
                                ..  respond correctly to a command           */
  BLD_NO_BOARD           = 17,  /* No board, or IO base address mismatch
                                .. between board and config file             */
  BLD_RESET_FAILED       = 18,  /* Driver could not find a board. I/O-address
                                .. or IRQ mismatch between board and values
                                .. supplied to the driver.                   */




  BLD_WRONG_CONTEXT      = 21,  /* Requested action is not valid in current
                                .. context. Maybe the function call sequence
                                .. was wrong                                 */
  BLD_ALREADY_ACTIVATED  = 22,  /* Activate request was issued by L1 was
                                .. already activated                         */
  BLD_ARCOFI_FAILED      = 23,  /* Arcofi did not respond to init commands   */
  BLD_DL_REL_IND         = 24,  /* Layer-2 DM message received               */
  BLD_MDL_ERR_IND        = 25,  /* Network does not respond                  */
  BLD_MDL_ERR_RESP       = 26,  /* Wrong response from network               */
  BLD_AUTO_MODE          = 27,  /* Operating in Auto mode                    */
  BLD_NON_AUTO_MODE      = 28,  /* NOT operating in Auto mode                */
  BLD_INVALID_KEY        = 29,  /* Invalid keypad key                        */
  BLD_BYTE_GEN_OCCUPIED  = 30,  /* B-channel byte generator occupied. Use    ..
                                .. value 0                                   */
  BLD_INVALID_BYTE_VAL   = 31,  /* Byte value must be less than 128          */
} BldRc;


typedef enum {
/*---------------------------------------------------------------------------*\
  Type of the receive/transmit frame (message).
\*---------------------------------------------------------------------------*/
  BLD_FM_HDLC,    /* Normal HDLC frame (including Layer 2)                   */
  BLD_FM_I,       /* I-frame (Only Layer 3, No Layer 2)                      */
  BLD_FM_UI,      /* UI-frame (Only Layer 3, No Layer 2)                     */
  BLD_FM_STR,     /* Information message (text) used to pass status and
                  .. debug information. Note: This has nothing to do with
                  .. the ISDN Layer 2 I-frames.                              */
  BLD_FM_UNDEF
} BldFrameType;


typedef enum {
/*---------------------------------------------------------------------------*\
  Operating modes for the Line Interface (Li).
\*---------------------------------------------------------------------------*/
  BLD_TE,         /* The line interface is configured to operate in Terminal
                  .. mode (as an TE).                                        */
  BLD_LT_S,       /* The line interface is configured to operate in Line
                  .. Termination mode (as an NT).                            */
  BLD_NO_IMODE
} LiMode;


typedef struct {
/*---------------------------------------------------------------------------*\
  Each received message is supplied with a data structure containing
  information on the received frame.
\*---------------------------------------------------------------------------*/
  int liNo;                  /* Number of the Line Interface which received
                             .. the message.                                 */
  unsigned char  fmSeqNo;    /* Sequence number. All the received messages
                             .. are assigned a sequence number by the driver.
                             .. After 256 frames the numbring is started
                             .. again from 0.                                */
  unsigned short fmLength;   /* Length of the received message.              */
  unsigned char  hour;       /* Time of reception: Hour                      */
  unsigned char  min;        /* Time of reception: Minute                    */
  unsigned char  sec;        /* Time of reception: Second                    */
  unsigned char  csec;       /* Time of reception: Hundreds of second        */
  BldFrameType   fmType;     /* Type of the received frame (HDLC or INFO)    */
  unsigned char  fmStatus;   /* Status of the received frame (???)           */
} BldFrameHeader;

/*---------------------------------------------------------------------------*\
  FUNCTION DECLARATIONS
\*---------------------------------------------------------------------------*/

char *bldIdentDriver(             // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Returns a pointer to an identification string for the driver.
    Useful for situations where the driver and the applications are
    not statically linked, where the application may want to report
    the revision or name of a currently dynamically linked driver.
\*---------------------------------------------------------------------------*/
void
);


BldRc bldConstructDriver(         // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Called to initialize the driver. This function needs to be
    executed before the driver can be used. The function verifies that
    supplied IRQ numbers and I/O addresses are valid and that the boards
    are installed (or seems to be installed, rather. It also installs
    the Interrupt Service Routines (ISRs), but not until disabling
    the IRQ from all LI's on all boards.

  RETURNS
    BLD_SUCCESS
    BLD_WRONG_CONTEXT
    BLD_INVALID_BOARD_TYPE
    BLD_INVALID_BOARD_NO
    BLD_INVALID_IRQ_NO
    BLD_INVALID_ADDR
    BLD_NO_BOARD
\*---------------------------------------------------------------------------*/
  int  boardType,       /* Type of the board used. Boardtype for Balder is
                        .. 102.                                              */
  int  boardBaseAddr,   /* I/0-base address wrapped on the board. Base
                        .. address for Balder boards is valid if:
                        .. (0x200 < Addr <= 0x400) && (Addr % 0x20) == 0     */
  int  noOfBoards,      /* Number of Boards installed.                       */
  int  irqTable[]       /* Table containing IRQs for all the boards installed
                        .. For example: if two boards with IRQs 10 and 11
                        .. have been installed, then irqTable should cantain
                        .. two items, 10 and 11. irqTable[] = { 10, 11 }     */
);


BldRc bldDestructDriver(          // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    This function must be called before the application is exited.
    Cleans up and clears the driver after use. Uninstalls the
    Interrupt Service Routines installed by the bldConstructDriver();

  RETURNS
    BLD_SUCCESS
    BLD_WRONG_CONTEXT
\*---------------------------------------------------------------------------*/
void
);

BldRc bldResetDriver(             // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Resets the Driver software. Clears the Receive and transmit memory
    buffers.

  RETURNS
    BLD_SUCCESS

  SEE ALSO
    BldResetLi()
\*---------------------------------------------------------------------------*/
  void
);


BldRc bldConfigureLi(             // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Sets up the Line Interface (ISAC-S) chip in either Terminal or
    Network mode.

  RETURNS
    BLD_INVALID_LI_NO
    BLD_NO_BOARD
    BLD_INVALID_MODE
    ..                  If none of the above then code from bldResetLi()
\*---------------------------------------------------------------------------*/
  int liNo,             /* Number of the Li chip to be configured            */
  LiMode liMode         /* Mode to be configured to, BLD_TE or BLD_LT_S      */
);



BldRc bldRead(                    // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Read the next received message. Driver will automatically sort the
    received messages from different Li according to the reception
    time. If several messages are waiting to be read, this function
    will read and return the oldest one.

  RETURNS
    BLD_SUCCESS   Frame read successfully
    BLD_NO_FRAMES if no complete frames have been received and ready to
                  be read.
\*---------------------------------------------------------------------------*/
  unsigned char   fmBuf[],   /* Buffer into which the received message will
                             .. be written. NOTE: The buffer must be
                             .. allocated by the application.                */
  int             fmBufSize, /* Size of fmBuf[].                             */
  BldFrameHeader *fmHeader   /* Pointer to header structure that will be
                             .. fillied in by the function. NOTE: The struct
                             .. must be allocated by the application.        */
);


BldRc bldWriteLi(                 // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Send an ISDN message including layer 2. Use bldWriteIframeLi() to
    send I-frames when operating in auto mode.

  RETURNS
    BLD_SUCCESS      // Message accepted and is being sent
    BLD_TX_BUSY      // Previous message not yet sent completely
                     // Try again later
\*---------------------------------------------------------------------------*/
  int             liNo,      /* Number of the Line Interface to be used for
                             .. sending.                                     */
  unsigned char   fmBuf[],   /* Buffer containing the message to be send.    */
  int             fmLength   /* Length of the message to be sent.            */
);

BldRc bldWriteIframeLi(                      // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Send an ISDN I-frame message in auto mode. Layer2 excluded (added
    automatically by Balder).

  RETURNS
    BLD_SUCCESS      // Message accepted and is being sent
    BLD_TX_BUSY      // Previous message not yet sent completely
                     // Try again later
\*---------------------------------------------------------------------------*/
  int             liNo,
  unsigned char   fmBuf[],
  int             fmLength
);

BldRc bldByteOnBch(               // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Start sending a constant Byte on the specified B-channel and keep sending
    it until turned off with the bldByteOffBch() function.

    The B-channel byte can have the value 0 - 127.

    If a non-zero byte is set on one B-channel then the other B-channels
    value is restricted to value 0.

  RETURNS
    BLD_SUCCESS

  SEE ALSO
    bldByteOffBch()
\*---------------------------------------------------------------------------*/
  int liNo,                  /* Number of the line interface to be used      */
  int bChannelNo,            /* Number of the B-channel to be used (1 or 2)  */
  unsigned char aByte        /* Byte to be send                              */
);


BldRc bldByteOffBch(              // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Stop the sending of a constant byte value on the specified B-channel, turn
    idle pattern (0xFF) on.

  RETURNS
    BLD_SUCCESS

  SEE ALSO
    bldByteOnBch()
\*---------------------------------------------------------------------------*/
  int liNo,                  /* Number of the line interface to be used      */
  int bChannelNo             /* Number of the B-channel to be used (1 or 2)  */
);


BldRc bldByteReadBch(             // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Reads one byte from the specified B-channel and returns it in recByte.

  RETURNS
    BLD_SUCCESS
\*---------------------------------------------------------------------------*/
  int liNo,                  /* Number of the line interface to be used      */
  int bChannelNo,            /* Number of the B-channel to be used (1 or 2)  */
  unsigned char *recByte     /* Received byte from the B-channel             */
);


BldRc bldGetStatusLi(             // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Read the status of an Line Interface.

   RETURNS
     BLD_L1_OK
     BLD_L1_DOWN
     BLD_UNDEFINED    // If LI is in LT_S mode in which case the sister
                      // LI holds the status. (Only happens in monitor mode)
\*---------------------------------------------------------------------------*/
  int liNo                   /* Number of the Line Interface to be read      */
);


BldRc bldResetLi(                 // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Perform reset on the HLDC for the Line Interface.

  RETURNS
    BLD_SUCCESS
    BLD_L1_DOWN

    BLD_RESET_FAILED
    BLD_NO_CLOCKS
    BLD_BAD_CHIP

  SEE ALSO
    bldResetDriver()
\*---------------------------------------------------------------------------*/
  int liNo              /* Number of the Line Interface to reset             */
);

BldRc bldChipResetLiPair(                  // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Perform hardware reset (whole chip) on the Line Interface. It also
    reconfigures the Li (if it was configured before reset). It takes
    two Li's as arguments. The first Li must have an even number, and
    the second Li number must be one higher than the first (i.e. an
    Li-pair).

  RETURNS
    code from bldConfigureLi()

  SEE ALSO
    bldResetLi()
\*---------------------------------------------------------------------------*/
  int    li1,
  int    li2
);

BldRc bldActivate(                // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Issues a Layer 1 Activate Request.

  RETURNS
    BLD_SUCCESS
    BLD_WRONG_CONTEXT

\*---------------------------------------------------------------------------*/
  int liNo              /* Number of the Line Interface to activate          */
);


BldRc bldDeactivate(              // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Issues a Layer 1 Deactivate Request.

  RETURNS
    BLD_SUCCESS
    BLD_WRONG_CONTEXT

\*---------------------------------------------------------------------------*/
  int liNo              /* Number of the Line Interface to deactivate        */
);


char *bldGetErrMsg(               // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Converts a Bld-function return code into a string describing the
    error. Returns a pointer to a static string owned by the function.

  RETURNS
    pointer to test string describing the errors code in
    a general fashion. Can be used for quick and dirty solutions
    when the return code is not analysed properly by the application
    but at least something needs to be printed.
\*---------------------------------------------------------------------------*/
  BldRc errCode         /* Error code to be converted                        */
);

BldRc bldPhoneOn(                 // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Connects a B-channel for a particular Li to the handset connector.

  RETURNS
    BLD_INVALID_LI_NO
    BLD_INVALID_B_CHANNEL
    BLD_ARCOFI_FAILED
    BLD_SUCCESS
\*---------------------------------------------------------------------------*/
  int liNo,             /* Number of the line interface to be used      */
  int bChannelNo        /* Number of the BCH to be used                 */
);



BldRc bldPhoneOff(                // funcdef
/*---------------------------------------------------------------------------*\
  SYNOPSIS
    Disconnects the B-channel for the indicated Li from the handset
    connector

  RETURNS
    BLD_INVALID_LI_NO
    BLD_INVALID_B_CHANNEL
    BLD_SUCCESS
\*---------------------------------------------------------------------------*/
  int liNo,             /* Number of the line interface to be used      */
  int bChannelNo        /* Number of the BCH to be used                 */
);




/*---------------------------------------------------------------------------*\
  SYNOPSIS

    Dials a phone number using DTMF tones on a B-channel. The time between
    tone can be specified in the 'silence' parameter, and the duration of
    each tone is determined by the valued of the 'duration' parameter.

   RETURNS

     BLD_SUCCESS
     BLD_INVALID_LI_NO
     BLD_INVALID_B_CHANNEL
     BLD_ARCOFI_FAILED

\*---------------------------------------------------------------------------*/
BldRc bldDtmfOnBch(
  int liNo,                 /* Number of the line interface to be used      */
  int bChannelNo,           /* Number of the BCH to be used                 */
  char *phoneNumber,        /* Phone number to dial                         */
  int duration,             /* Duration of DTMF tones in clock ticks (0 for ..
                            ..  (0 for infinity)                            */
  int silence               /* Silence time between of DTMF tones in clock  ..
                            .. clock ticks                                  */
);

/*---------------------------------------------------------------------------*\
  SYNOPSIS

    Software reset of the Codec.

   RETURNS

     BLD_ARCOFI_FAILED
     BLD_SUCCESS

\*---------------------------------------------------------------------------*/
BldRc bldResetCodec(
  int liNo                   /* Number of the line interface to be used      */
);

/*---------------------------------------------------------------------------*\
  SYNOPSIS


   RETURNS


\*---------------------------------------------------------------------------*/
BldRc bldFifoUsage(
  int liNo,                  /* Number of the line interface to be used      */
  unsigned int *rxHdlcFifo,
  unsigned int *rxInfoFifo,
  unsigned int *txHdlcFifo
);


#ifdef __cplusplus
  } // extern "C"
#endif

#endif   /* BALDER_H */
